/**
 * @file filter.c Message filter utility functions
 *
 * Copyright (C) 2004-2005 Christian Hammond.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA  02111-1307  USA
 */
#include "filter.h"
#include "utils.h"

void
galagod_dbus_register_object_path(const char *path,
								  const DBusObjectPathVTable *vtable,
								  void *user_data)
{
#if DBUS_CHECK_VERSION(0, 22)
	if (!dbus_connection_register_object_path(galago_core_get_dbus_conn(), path,
										 vtable, user_data))
		abort();
#else
	char **path2;

	path2 = g_strsplit(path + 1, "/", -1);

	dbus_connection_register_object_path(galago_core_get_dbus_conn(),
										 (const char **)path2, vtable,
										 user_data);

	g_strfreev(path2);
#endif
}

void
galagod_dbus_unregister_object_path(const char *path)
{
#if DBUS_CHECK_VERSION(0, 22)
	dbus_connection_unregister_object_path(galago_core_get_dbus_conn(), path);
#else
	char **path2;

	path2 = g_strsplit(path + 1, "/", -1);

	dbus_connection_unregister_object_path(galago_core_get_dbus_conn(), path2);

	g_strfreev(path2);
#endif
}

static void
path_unregistered_func(DBusConnection *dbus_conn, void *user_data)
{
	g_free(user_data);
}

static DBusHandlerResult
path_message_func(DBusConnection *dbus_conn, DBusMessage *message,
				  void *user_data)
{
	GalagodStateData *state_data = (GalagodStateData *)user_data;

	return state_data->func(dbus_conn, message, state_data);
}

static DBusObjectPathVTable common_vtable =
{
	path_unregistered_func,
	path_message_func,
	NULL
};

void
galagod_filters_add(void *obj, DBusHandleMessageFunction func)
{
	GalagodStateData *state_data;
	GalagodCoCo *coco;
	GalagoContext *context;

	galago_return_if_fail(obj  != NULL);
	galago_return_if_fail(func != NULL);
	galago_return_if_fail(GALAGO_IS_OBJECT(obj));

	coco = GALAGOD_COCO(obj);

	context = (coco != NULL
			   ? galagod_coco_get_context(coco)
			   : galagod_get_context());

	state_data = g_new0(GalagodStateData, 1);
	state_data->coco     = GALAGOD_COCO(obj);
	state_data->context  = context;
	state_data->u.object = obj;
	state_data->func     = func;

	galagod_dbus_register_object_path(galago_object_get_dbus_path(obj),
									  &common_vtable, state_data);
}

void
galagod_filters_remove(void *obj)
{
	galago_return_if_fail(obj != NULL);
	galago_return_if_fail(GALAGO_IS_OBJECT(obj));

	galagod_dbus_unregister_object_path(galago_object_get_dbus_path(obj));
}
